// $RCSfile: MPClient.java,v $
// $Author: aboyer $ 
// $Date: 2004/02/25 14:32:31 $
// $Revision: 1.18 $

package com.spamnbeans.client;

import com.spamnbeans.server.*;

import javax.naming.InitialContext;
import javax.naming.Context;
import javax.rmi.PortableRemoteObject;

import java.util.logging.*;
import java.io.*;
import java.util.Hashtable;

/**
 * Spam-n-Beans mail processing system client interface.
 * <p>
 * This client interface connects to a Mail Processing Server and submits an
 * email for processing.  The email is read from standard input, and a copy of
 * the email message is printed on standard output along with any relevant
 * changes.  It will attempt to retry several times if errors occur.
 *
 * @author Andrew Boyer,
 *			<a href="mailto:aboyer@andrew.cmu.edu">aboyer@andrew.cmu.edu</a>
 * @version 1.0
 *
 * @author Philippe M. Wilson
 *			&lt;<a href="mailto:pmwilson@cmu.edu">pmwilson@cmu.edu</a>&gt;
 * @version 1.1
 */
public class MPClient
{
	private static Logger logger = Logger.getLogger(MPClient.class.getName());

	private static void setInactive( ManagerRemote mgr, String name )
	{
		try {
			mgr.serverSetInactive(name);
		} catch (Throwable t) { }
	}

	public static void main(String [] args)// throws Throwable
	{
		logger.info("Running end-to-end use case...");

		String header = null;
		String body = null;
		String msg = null;
		try {  // Grab an input message from the console
			BufferedReader in = new BufferedReader(new 
					InputStreamReader(System.in));
			StringBuffer buf = new StringBuffer();
			String inputLine;
			while ((inputLine = in.readLine()) != null) {
				if (header == null && inputLine.length() == 0) {
					header = buf.toString();
					buf = new StringBuffer();
				}
				buf.append(inputLine);
				buf.append("\r\n");
			}
			body = buf.toString();


			in.close();

			if (header == null) {
				// the entire message is formatted like a header
				header = body;
				body = "";
			}

			msg = header + body;
			if (msg.length() == 0) {
				logger.severe( "Input message cannot be of length 0");
				System.exit(2200);
			}
		}
		catch (Throwable t) {
			if (logger.isLoggable(Level.FINEST)) {
				logger.log(Level.FINEST, "Error reading input message:", t);
			}
			else {
				logger.severe("Failed to read input message");
			}
			System.exit(2100);
		}

		ManagerHomeRemote mgrhome = null;
		ManagerRemote mgr = null;
		logger.info("Connecting to Naming Service...");
		try {  // Set up a connection to the golden name server
			Context jndiContext = new InitialContext();

			Object mgrref = jndiContext.lookup("ManagerHomeRemote");

			logger.info("Connected to Naming Service (JNDI)");

			mgrhome = (ManagerHomeRemote)
				PortableRemoteObject.narrow(mgrref,ManagerHomeRemote.class);
		}
		catch (Throwable t) {
			if (logger.isLoggable(Level.FINEST)) {
				logger.log(Level.FINEST, "Error connecting to Naming Service:", t);
			} else {
				logger.warning("Error connecting to Naming Service");
			}
			logger.severe("Failed to connect to the Naming Service");
			System.exit(100);
		}


		logger.info("Connecting to a Processing Server...");
		boolean successMPServer=false;
		for(int i=0; i<8 && successMPServer==false; i++){
			MailProcHomeRemote mphome = null;
			MailProcRemote mp = null;
			String result = null;

			// connect to replication manager
			boolean successRepMan=false;
			while(successRepMan==false){
				try {
					mgr = mgrhome.create();

					logger.info("Connected to Load Balancer (Replication Manager)");
					successRepMan=true; // DONE, move on to next step
				} catch (Throwable t) {
					if (logger.isLoggable(Level.FINEST)) {
						logger.log(Level.FINEST, "Error connecting to Load Balancer (Replication Manager)", t);
					} else {
						logger.warning("Error connecting to Load Balancer (Replication Manager)");
					}
					break; // don't retry
				}
			}
			if(successRepMan==false)
				continue; // retry

			// connect to replicated processing server
			boolean successRepMPServer=false;
			for(int j=0; j<2 && successRepMPServer==false; j++){ // try a few times
				try {
					String mpname = mgr.serverLoadBalance();

					logger.info("Directing request to "+mpname+" ...");

					Context jndiContext = new InitialContext();
					jndiContext.addToEnvironment("java.naming.provider.url", mpname);
					Object mpref = jndiContext.lookup("MailProcHomeRemote");

					logger.info("Connecting to Processing Server...");

					mphome = (MailProcHomeRemote)
						PortableRemoteObject.narrow(mpref,MailProcHomeRemote.class);

					mp = mphome.create();
					mp.attachServer(mpname);

					logger.info("Connected to Processing Server: " + mp.getServerName());
					successRepMPServer=true; // DONE, move on to next step
				} catch (Throwable t) {
					if (logger.isLoggable(Level.FINEST)) {
						logger.log(Level.FINEST, "Error connecting to Processing Server", t);
					} else {
						logger.warning("Error connecting to Processing Server");
					}
				}
			}
			if(successRepMPServer==false)
				continue; // retry

			
			// send message for processing
			boolean successProcess=false;
			for(int j=0; j<2 && successProcess==false; j++){ // try a few times
				try{
					result = mp.processMessage(new Integer(args[0]),
							new Integer(msg.hashCode()), msg);	
					System.out.print(header + result + body);
					System.exit(0);
				}catch (InvalidClientException ice) {  // Client rejected
					logger.severe("Invalid Client ID");
					System.exit(2300);
				} catch (Throwable t) {
					if (logger.isLoggable(Level.FINEST)) {
						logger.log(Level.FINEST, "Error processing message:", t);
					}
					else {
						logger.warning("Failed to process message");
					}
					continue;
				}
			}

			successMPServer=true; // successful connection to a Mail Processing Server
		}
		if(successMPServer==false){
			logger.severe("Failed to connect to a Processing Server");
			System.exit(110);

		}



// SEND MESSAGE






			mgr = mgrhome.create();
			logger.info( "Connected to name server" );
		}
		catch (Throwable t) {
			if (logger.isLoggable(Level.FINEST)) {
				logger.log(Level.FINEST, "Error connecting to server", t);
			}
			else {
				logger.severe("Error connecting to server");
			}
			System.exit(1100);
		}

		while (true) {
			String mpname = null;

			try {
				// Contact one of the pool of servers
				mpname = mgr.serverLoadBalance();
				if ( mpname == null )
					throw new InvalidServerException( "Null server" );

				logger.info( "Attempting to contact " + mpname );

				Context jndiContext = new InitialContext();
				jndiContext.addToEnvironment( "java.naming.provider.url", 
						mpname );
				Object mpref = jndiContext.lookup( "MailProcHomeRemote" );

				MailProcHomeRemote mphome = (MailProcHomeRemote)
					PortableRemoteObject.narrow(mpref,
							MailProcHomeRemote.class);
				MailProcRemote mp = mphome.create();
				mp.attachServer( mpname );

				logger.info( "Connected to " + mp.getServerName() );

				// Check the message
				String result = mp.processMessage( new Integer( args[0] ),
						new Integer(msg.hashCode()), msg);
				System.out.print(header + result + body);
				System.exit(0);
			}
			catch (InvalidClientException ice) {  // Client rejected FATAL
				logger.severe("Invalid Client ID");
				System.exit(2300);
			}
			catch (InvalidServerException ise) { // No more servers FATAL
				if (logger.isLoggable(Level.FINEST)) {
					logger.log(Level.FINEST, "No servers found", ise);
				}
				else {
					logger.severe( "No servers found" );
				}
				System.exit(666);
			}
			catch (ProcessingFailureException pfe) { // Non-fatal failover
				System.out.println( pfe.getMessage() );

				if (logger.isLoggable(Level.FINEST)) {
					logger.log( Level.FINEST, "Error processing message on " +
							mpname, pfe );
				}
				else {
					logger.severe( "Error processing message on " + mpname );
				}
				setInactive( mgr, mpname );
			}
			catch (Throwable t) {  // Unspecified non-fatal failover
				System.out.println( t.getMessage() );
				if (logger.isLoggable(Level.FINEST)) {
					logger.log( Level.FINEST, "Error connecting to server " +
							mpname, t );
				}
				else {
					logger.severe( "Error connecting to server " + mpname );
				}
				setInactive( mgr, mpname );
			}
		}
	}
}
