#!/usr/bin/perl -w
##########################################################################
# $Id: Multi-Client.pl,v 1.6 2004/02/14 03:29:00 gca Exp $
#
# Description: spawns multiple simulatneously running Spam'n'Beans clients
##########################################################################

use strict;

use Thread; 		# For multi-threading...
use File::Temp;		# To create temporary output files
use File::Compare;	# For comparing original & resultant files
use File::Copy;     # to copy file in O/S independent way

# Timeout (in seconds) for clients, after which processing is aborted.
my $CLIENT_TIMEOUT = 5;

# directories
#my $base = '/afs/ece/usr/cfry/classes/middleware';
#my $publiccorpus = "$base/publiccorpus";
my $TestMsg = "Original.msg";

# Subroutine prototypes
sub SetupClientIDs($);
sub SpawnClients($);

#-------------------------------------------------------------------------
main: {
  print "Running Multi-Client...\n";
  
  my $NbrClientsToSpawn = 3;
  
  # Copy test message to local dir
  File::Copy::copy("./scripts/test1.spam","Original.msg");
  
  # Check input test message
  my @msg;
  open IN, "$TestMsg";
  chomp (@msg = <IN>) or 
    die "ERROR! - unable to read $TestMsg\n$!";
  close IN;
  print "Test Message $TestMsg, Length:$#msg\n";
  ($#msg > 0) or die "ERROR! - InvalidMsgLength:$#msg\n$!";
  
  SetupClientIDs($NbrClientsToSpawn) or 
    die "ERROR - Unable to setup Clients!\n$!";
  my $test_result = SpawnClients($NbrClientsToSpawn);
  
  # Check test results
  # - Translate return success indicator to client
  #   - 0 := success for ant or other shell scripts
  if ( $test_result == 1 ) {
    print "Test Complete - PASS!\n";
  } else {
    die "Test Complete - FAIL!\n";
  }

}

#-------------------------------------------------------------------------
# Creates the number of client ids specified by the caller
# - Params : The number of clients to create
#   - Simply creates sequential id's, started from 1
# - Return : a boolean value, true for success
# 
sub SetupClientIDs($) {
   my $NbrClientsToCreate = shift;
   my $mgrclient = "java com.spamnbeans.client.MPManager";
   my $rslt;
   
   ( $NbrClientsToCreate > 0 ) or 
      die "ERROR - Invalid Nbr of clients to spawn! \n$!";
  
   # Remove any existing ClientsIDs (and ClientStats) & Create new ones...
   print "Removing existing ClientIDs and Creating new ones...\n";
   my $cid;  # The client ID
   for( $cid = 1; $cid <= $NbrClientsToCreate; $cid++  ) {
      print "Client[$cid]";
      my $rslt = `$mgrclient client exists $cid`;
      if( $rslt =~ m/^TRUE/ ) {
         print "\t Already Exists! - Deleting...";
         $rslt = `$mgrclient client remove $cid`;
         $rslt =~ m/^Done/ or
           die "$rslt\nERROR! - Unable to remove Client[$cid]!\n$!";
      }
      print "\tCreating...";
      $rslt = `$mgrclient client create $cid`;
      $rslt =~ m/^Done/ or
         die "$rslt\nERROR! - Unable to create Client[$cid]!\n$!";
      print "\tOk\n";
   }
   
   return 1;
}

#-------------------------------------------------------------------------
# Checks for presense of XSpamHeader in message
# - Params : The message to process
# - Return : a boolean value, true if X-Spam header exists
# 
sub ContainsXSpamHeader(@) {
  my $fContainsSpamHeader = 0;
  
  # Check for valid msg
  ($#_ > 0) or die "ERROR! - InvalidMsgSize:$#_\n$!";
  
  foreach (@_) {
#    print $_;
    if (m/^X-Spam-Level: ?(\**)$/) {
      $fContainsSpamHeader = 1;
      last;
    }
  }
  return $fContainsSpamHeader;
}

#-------------------------------------------------------------------------
# The child-process spawned by 'SpawnClients'
# - Params : The file containing the message to process
# - Return : the modified message
# 
sub ClientProcess(@) {
  my $OriginalMsg = shift;
  my @ReturnedMsg;				# the return value for this thread
  my $tid = Thread->self->tid;	# the thread-id
  my $m = "ClientProcess[$tid]";# print prefix for this thread
  print "$m Started\n";
  
  # Identify the client executable and arguments
  # - use thread-id as client-id   
  my $cmd = "java com.spamnbeans.client.MPClient $tid < $OriginalMsg";
  
  # Spawn the client & measure how long it take it to run
  # - In this case, client returns the modified message
  print "$m Processing... $cmd\n";
  my $start_time = time;
  if( 1 ) { # do the real thing
    @ReturnedMsg = `$cmd`;
  } elsif ( 1 ) {
    # Debug - Simulate returned message by echoing back original
    open IN, "$OriginalMsg";
    chomp (@ReturnedMsg = <IN>);
    close IN;
  } elsif ( 1 ) {
    # For debug - simulate a valid header
    @ReturnedMsg = ("foobar\n","X-Spam-Level: (*10)\n");
  }
  my $elapsed_time = time - $start_time; 
  
  printf "$m Terminated - Execution Time:%d\n",$elapsed_time;
  return @ReturnedMsg;
}

#-------------------------------------------------------------------------
# Creates child-processes
# - Params : The number of child processes to spawn
# - Return : a boolean value : true = success
#   - indicating aggregate client success/failure
# 
sub SpawnClients($) {
  my $NbrClientsToSpawn = shift;
  
  if( $NbrClientsToSpawn <= 0 ) {
    die "ERROR! - Invalid Nbr of clients to spawn! \n$!";
    return 0;
  }
  
  my @client_args = "$TestMsg";
  my @t;  # array of thread handles
  my @r;  # array of thread results
  
  # Spawn child processes
  # - allow each to run simultaneously
  print "Spawning Clients...\n";
  for( my $i = 1; $i <= $NbrClientsToSpawn; $i++  ) {
    @t = (@t, Thread->new(\&ClientProcess, @client_args) );
  }
  
  # Wait for each each child processes to finish
  # - as they finish, check their results
  print "Waiting for Clients to finish...\n";
  my $rslt = 1;	# assume success
  foreach my $t ( @t ) {
    @r = $t->join;	# The modifed msg (stdout) from the client
    
    # Record the results to a tmpfile
    my $tmpfile = "ModifiedMsg.txt";
    open FH, ">$tmpfile" or
      die "ERROR! - Unable to open $tmpfile\n$!";
    foreach (@r) {
      print FH $_ or 
        die "ERROR! - Unable to write to $tmpfile!\n$!";
    }
    close FH;
    
    # Compare input message against returned message
    # - obtain diff of results
    # - Note: This confirms body of message is not altered
    my $compare = File::Compare::cmp($TestMsg,$tmpfile);
    if ( $compare == 0 ) {
      # No difference, SnB failed to add X-Spam headers
      print "ERROR! - results identical to original!\n";
      $rslt = 0;
    } elsif( $compare == -1 ) {
      die "ERROR! - error comparing results!\n";
    } elsif( $compare == 1 ) {
      # Validate that difference is that returned message 
      # only has an added X-Spam header
      # - if more differences than X-Spam headers, then SnB
      #   incorrectly modified the message body.
      if( ContainsXSpamHeader(@r) == 0) {
        print "$t ERROR! ReturnedMsg lacks X-Spam header!\n";
        $rslt = 0;
        next;
      }
      # Need to confirm that if spam headers removed...
      # ...then message is identical to original
    }
    
  }
  
  return $rslt;  # return aggregate results to caller
}
