// $RCSfile: MPManager.java,v $
// $Author: aboyer $ 
// $Date: 2004/03/21 22:46:31 $
// $Revision: 1.19 $

package com.spamnbeans.client;

import com.spamnbeans.server.*;

import javax.naming.InitialContext;
import javax.naming.Context;
import javax.rmi.PortableRemoteObject;
import java.lang.ArrayIndexOutOfBoundsException;

import java.util.*;
import java.net.*;

/**
 * Management Client for Spam-n-Beans mail processing system.
 * <p>
 * This client interface connects to a Management Server and enables the user
 * to perform management activities such as: creating, removing, modifying and
 * listing clients and servers.
 * <p>
 * Use:  <item> <operation>
 * where: <item> ::= server | client | msg
 *        <operation> :: = create <id | hostname | ip> <name> | # Create the item id/name
 *  			     remove <id> |  # Remove the item with id
 * 			     exists <id> |  # Test if the item with id exists
 * 			     getname <id> | # Get name of item with id
 * 			     getid <name> | # Get id of item with name
 * 			     list | # List all the id's of the type
 * 			     clientStats <id> # List aggregate stats per client
 * 	    <id> ::= [0-9] 	  # ID's are simple integers
 *	    <name> ::= [a-z,0-9]  # Names are simple alpha-numeric strings
 *
 * @author Andrew Boyer,
 *			<a href="mailto:aboyer@andrew.cmu.edu">aboyer@andrew.cmu.edu</a>
 * @version 1.0
 *
 * @author Philippe M. Wilson
 *			&lt;<a href="mailto:pmwilson@cmu.edu">pmwilson@cmu.edu</a>&gt;
 * @version 1.2
 *
 */
public class MPManager {
	public static void main(String[] args) throws Exception {
		ManagerRemote mgr;

		//System.out.println( "Running SpamNBeans manager..." );

		try { // Set up a connection to the server
			Context jndiContext = new InitialContext();
			Object ref = jndiContext.lookup("ManagerHomeRemote");
			ManagerHomeRemote home = (ManagerHomeRemote)
				PortableRemoteObject.narrow(ref, ManagerHomeRemote.class);

			mgr = home.create();
		} catch (Throwable t) {
			System.out.println("Error connecting to server: " + t);
			return;
		}

		String server = "server";
		String client = "client";
		String msg = "msg";

		String create = "create";
		String remove = "remove";
		String flush = "flush";
		String exists = "exists";
		String getname = "getname";
		String getid = "getid";
		String enable = "enable";
		String disable = "disable";
		String getenable = "getenable";
		String listenabled = "listenabled";
		String list = "list";
		String clientstats = "clientstats";

		try { // Parse input arguments
			if (args[0].equals(server)) {
				// Process server commands
				if (args[1].equals(create)) {
					mgr.serverCreate(new Integer(args[2]), args[3], true);
					System.out.println(
						"Created server " + args[3] + " [" + args[2] + "]");
				} else if (args[1].equals(remove)) {
					mgr.serverRemove(new Integer(args[2]));
					System.out.println("Removed server [" + args[2] + "]");
				} else if (args[1].equals(exists)) {
					if (mgr.serverExists(new Integer(args[2])))
						System.out.println(
							"TRUE: Server [" + args[2] + "] exists");
					else
						System.out.println(
							"FALSE: Server [" + args[2] + "] does not exist");
				} else if (args[1].equals(getname)) {
					System.out.println(
						"Server ["
							+ args[2]
							+ "] is named "
							+ mgr.serverGetName(new Integer(args[2])));
				} else if (args[1].equals(getid)) {
					System.out.println(
						"Server named "
							+ args[2]
							+ " has ID ["
							+ mgr.serverGetID(args[2])
							+ "]");
				} else if (args[1].equals(getenable)) {
					String strMsg ="Server " + args[2] + " is enabled:" 
						+ new Boolean(mgr.serverIsEnabled(args[2])).toString();
					System.out.println(strMsg);
				} else if (args[1].equals(enable)) {
					mgr.serverSetEnabled(args[2],true);
					System.out.println("Server " + args[2] + " is now enabled");
				} else if (args[1].equals(disable)) {
					mgr.serverSetEnabled(args[2],false);
					System.out.println(
						"Server " + args[2] + " is now disabled");
				} else if (args[1].equals(listenabled)) {
					System.out.println("Servers enabled:");
					HashSet servers = mgr.serverListEnabled();
					Iterator it = servers.iterator();
					while (it.hasNext()) {
						String n = (String) it.next();
						System.out.println(n + " [" + mgr.serverGetID(n) + "]");
					}
				} else if (args[1].equals(list)) {
					System.out.println("Servers registered:");
					HashSet servers = mgr.serverListIDs();
					Iterator it = servers.iterator();
					while (it.hasNext()) {
						Integer s = (Integer) it.next();
						System.out.println(s + ": " + mgr.serverGetName(s));
					}
				} else if (args[1].equals(flush)) {
					HashSet servers = mgr.serverListIDs();
					System.out.println(
						"Flushing " + servers.size() + " servers");
					Iterator it = servers.iterator();
					while (it.hasNext()) {
						Integer s = (Integer) it.next();
						mgr.serverRemove(s);
					}
				} else
					throw new ArrayIndexOutOfBoundsException();
			} else if (args[0].equals(client)) {
			    // Process client commands
			    Integer clientID = new Integer(0);
			    String cID = "";
			    if (args.length > 2) {
				clientID = MPClient.getClientID(args[2]);
				cID = args[2]+" ("+clientID.toString()+")";
			    }

			    if (args[1].equals(create)) {
				mgr.clientCreate(clientID, args[2]);
				System.out.println(
						   "Created client " + args[2] + " [" + cID + "]");
			    } else if (args[1].equals(remove)) {
				mgr.clientRemove(clientID);
				System.out.println("Removed client [" + cID + "]");
			    } else if (args[1].equals(exists)) {
				if (mgr.clientExists(clientID))
				    System.out.println(
						       "TRUE: Client [" + cID + "] exists");
				else
				    System.out.println(
						       "FALSE: Client [" + cID + "] does not exist");
			    } else if (args[1].equals(getname)) {
				System.out.println(
						   "Client ["
						   + cID
						   + "] is named "
						   + mgr.clientGetName(clientID));
			    } else if (args[1].equals(getid)) {
				System.out.println(
						   "Client named "
						   + cID
						   + " has ID ["
						   + mgr.clientGetID(args[2])
						   + "]");
			    } else if (args[1].equals(list)) {
				System.out.println("Clients running:");
				HashSet clients = mgr.clientListIDs();
				Iterator it = clients.iterator();
					while (it.hasNext()) {
						Integer s = (Integer) it.next();
						System.out.println(s + ": " + mgr.clientGetName(s));
					}
				} else if (args[1].equals(flush)) {
					HashSet clients = mgr.clientListIDs();
					System.out.println(
						"Flushing " + clients.size() + " clients");
					Iterator it = clients.iterator();
					while (it.hasNext()) {
						Integer c = (Integer) it.next();
						mgr.clientRemove(c);
					}
				} else
					throw new ArrayIndexOutOfBoundsException();
			} else if (args[0].equals(msg)) {
				// Process msg commands
				if (args[1].equals(clientstats)) {
					Integer clientID = MPClient.getClientID(args[2]);
					String cID = args[2]+" ("+clientID.toString()+")";
					System.out.println(
						"Client ["
							+ cID
							+ "] stats:\n"
							+ mgr.msgGetStatsByClient(clientID));
					System.out.println("Done");
				} else if (args[1].equals(flush)) {
					HashSet msgs = mgr.msgListIDs();
					System.out.println("Flushing " + msgs.size() + " messages");
					Iterator it = msgs.iterator();
					while (it.hasNext()) {
						Integer m = (Integer) it.next();
						mgr.msgRemove(m);
					}
				} else {
					throw new ArrayIndexOutOfBoundsException();
				}
			} else {
				throw new ArrayIndexOutOfBoundsException();
			}
		} catch (ArrayIndexOutOfBoundsException ae) {
			System.out.println(""
					+ "Usage: [server] [create int string]\n"
					+ "                [remove int]\n"
					+ "                [exists  int]\n"
					+ "                [getname int]\n"
					+ "                [getid string]\n"
					+ "                [list]\n"
					+ "                [flush]\n"
					+ "                [enable string]\n"
					+ "                [disable string]\n"
					+ "                [getactive string]\n"
					+ "                [listenabled]\n"
					+ "\n"
					+ "       [client] [create string]\n"
					+ "                [remove string]\n"
					+ "                [exists string]\n"
					+ "                [getname int]\n"
					+ "                [getid string]\n"
					+ "                [list]\n"
					+ "                [flush]\n"
					+ "\n"
					+ "       [msg]    [clientstats int]\n"
					+ "                [flush]\n");
		} catch (Throwable t) {
			System.out.println("Error: " + t.getMessage());
		}
	}

}
