// $Id: GlobalReplicationManager.java,v 1.16 2004/03/23 01:58:54 pmwilson Exp $
package com.spamnbeans.repmgr;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Properties;
import java.util.*;

import javax.naming.InitialContext;
import javax.naming.Context;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;

import com.spamnbeans.server.*;

import java.util.logging.*;

/**
 * The Global Replication Manager
 * <p>
 * Offers system administrators a menu-driven console to control and monitor of
 * the entire Spam'n'Beans system
 * <p>
 * Spawns seperate threads devoted to polling each replica
 * 
 * @author Gary Ackley, <a href="mailto:gca@andrew.cmu.edu">gca@andrew.cmu.edu
 *         </a>
 * @version $Revision: 1.16 $
 */
public class GlobalReplicationManager implements Runnable {
	private static Logger logger = Logger.getLogger(GlobalReplicationManager.class.getName());
	protected int sleepPeriod;
	protected int numServers = 0;
	static ArrayList replicas = new ArrayList();
	static Context jndiContext;

	public GlobalReplicationManager(int period) {
		System.out.println("Creating GlobalReplicationManager...");
		this.sleepPeriod = period;
	}

	/**
	 * main - The main startup routine
	 * 
	 * @param args -
	 *            Replica poll rate
	 */
	public static void main(String[] args) {
		// Setup connection to local JNDI server
		try {
			/*
			 * Setup properties for JNDI Initialization - These are read from
			 * command line - This may be used for such things as Security,
			 * limiting access to a bean and throwing SecurityExceptions if not
			 * granted.
			 */
			Properties props = System.getProperties();

			/*
			 * Obtain the JNDI Initial Context - This is the starting point for
			 * contacting the JNDI Tree. We choose or JNDI driver, the network
			 * location of the server, etc. by passing in the environment
			 * properties
			 */
			jndiContext = new InitialContext(props);

			/*
			 * Since we now have a context, we may query it for special
			 * environment details... - Such details are often embedded into
			 * the deployment descriptor
			 */
			//String userName =
			//	(String)jndiContext.lookup("java:comp/User");
			//System.out.println("Welcome " + userName);
		} catch (Throwable t) {
			System.out.println("Error connecting to local JNDI\n" + t);
			return;
		}

		// Parse any startup parameters
		int checkPeriod = 60;
		switch (args.length) {
			case 0 : // No parameters supplied
				break;
			case 1 : // One parameter - the checkPeriod
				int newCheckPeriod = Integer.parseInt(args[0]);
				System.out.println(
						"Overriding default PollPeriod:"
						+ checkPeriod
						+ "-->"
						+ newCheckPeriod);
				checkPeriod = newCheckPeriod;
				break;
			default :
				ShowUsage();
				return;
		}

		// Invoke the Replication Manager as a deamon in it's own thread
		GlobalReplicationManager grm =
			new GlobalReplicationManager(checkPeriod);
		grm.run();
	}

	/**
	 * Run - The Spam'n'Beans console
	 */
	public void run() {
		System.out.println("Starting GlobalReplicationManager Console...");

		//distribute the server assignments
		MenuMain();
	}

	private static void ShowUsage() {
		String strMsg =
			"Usage: GlobalReplicationManager [checkPeriod]\n"
			+ "\tcheckPeriod -- seconds to sleep between server polls "
			+ "(default=60)";
		System.out.println(strMsg);
	}

	private void MenuMain() {
		// Display and handle the management menu(s)
		String strMainMenu =
			"\nMain Menu:\n"
			+ "1) Manage Servers (Replicas)\n"
			+ "2) Manage Clients\n"
			+ "3) Manage Messages\n"
			+ "4) Shutdown System & Exit\n"
			+ "Choice:";

		do {
			System.out.print(strMainMenu);

			try {
				// Read user input
				BufferedReader in =
					new BufferedReader(new InputStreamReader(System.in));
				String choice = in.readLine();

				// Select indicated submenu - or terminate
				switch (Integer.parseInt(choice)) {
					case 1 :
						MenuServer();
						break;
					case 2 :
						MenuClient();
						break;
					case 3 :
						MenuMsg();
						break;
					case 4 :
						ServerStartStop(new Integer(-1), false);
						System.out.println("Terminating Console...");
						return;
					default :
						throw new NumberFormatException();
				}
			} catch (NumberFormatException e) {
				System.out.println("Enter a Valid Number & Try Again...");
			} catch (IOException e) {
				String strMsg = "Internal Error - ";
				System.out.println(strMsg+e);
				logger.log(Level.WARNING, strMsg, e);
			}

			Thread.yield();
		} while (true);
	}

	private void MenuServer() {
		// Display and handle the management menu(s)
		String strMsg = ""
			+ "\nServer/Replica Menu:\n"
			+ "1) List Servers\n"
			+ "2) Start Server\n"
			+ "3) Stop Server\n"
			+ "4) Add Server\n"
			+ "5) Remove Server\n"
			+ "Choice:";
		System.out.print(strMsg);

		try {
			BufferedReader in =
				new BufferedReader(new InputStreamReader(System.in));
			String choice = in.readLine();

			switch (Integer.parseInt(choice)) {
				case 1 : // List Servers & Replica Status
					ServerList();
					break;
				case 2 : // Enable Server/Replica
				    ServerRun(true); // enable server
				    break;
				case 3 : // Disable Server/Replica
				    ServerRun(false); // disable server
				    break;
				case 4 : // Add Server
					ServerCreate();
					break;
				case 5 : // Remove Server
					ServerRemove();
					break;
				default :
					throw new NumberFormatException();
			}
		} catch (NumberFormatException e) {
			System.out.println("Enter a Valid Number & Try Again...");
		} catch (IOException e) {
			strMsg = "Error reading input";
			logger.log(Level.WARNING, strMsg, e);
		}
	}

	private void MenuClient() {
		// Display and handle the management menu(s)
		String strClientMenu =
			"\nClient Menu:\n"
			+ "1) List Clients\n"
			+ "2) Enable Client\n"
			+ "3) Disable Client\n"
			+ "4) Add Client\n"
			+ "5) Remove Client\n"
			+ "Choice:";
		System.out.print(strClientMenu);
		try {
			BufferedReader in =
				new BufferedReader(new InputStreamReader(System.in));
			String choice = in.readLine();

			switch (Integer.parseInt(choice)) {
				case 1 : // List Clients
					ClientList();
					break;
				case 4 : // Add Client
					ClientCreate();
					break;
				case 5 : // Remove Client
					ClientRemove();
					break;
				default :
					System.out.println("Sorry - Not Implemented Yet...");
					throw new NumberFormatException();
			}
		} catch (NumberFormatException e) {
			System.out.println("Enter a Valid Number & Try Again...");
		} catch (IOException e) {
			String strMsg = "Error reading input";
			logger.log(Level.WARNING, strMsg, e);
		}
	}

	private void MenuMsg() {
		// Display and handle the management menu(s)
		String strMsgMenu =
			"\nMsg Menu:\n"
			+ "1) Show Global Message Statistics\n"
			+ "2) Show Aggregate Message Statistics per Client\n"
			+ "3) Show Aggregate Message Statistics per Server\n"
			+ "4) List Messages per Client\n"
			+ "5) List Messages per Server\n"
			+ "6) Display Message\n"
			+ "7) Add Message\n"
			+ "8) Remove Message\n"
			+ "Choice:";
		System.out.print(strMsgMenu);
		try {
			BufferedReader in =
				new BufferedReader(new InputStreamReader(System.in));
			String choice = in.readLine();

			switch (Integer.parseInt(choice)) {
				default :
					System.out.println("Sorry - Not Implemented Yet...");
					throw new NumberFormatException();
			}
		} catch (NumberFormatException e) {
			System.out.println("Enter a Valid Number & Try Again...");
		} catch (IOException e) {
			String strMsg = "Error reading input";
			logger.log(Level.WARNING, strMsg, e);
		}
	}

	// List all servers & their status
	private void ServerList() {
		String strObjName = "ManagerHomeRemote";
		String strMsg;

		try {
			// Get the Remote ManagerBean
			Object ref = jndiContext.lookup(strObjName);
			ManagerHomeRemote home =
				(ManagerHomeRemote) PortableRemoteObject.narrow(
																ref,
																ManagerHomeRemote.class);
			ManagerRemote mgr = home.create();

			// List set of all known servers
			// - If server is active, show replica status
			strMsg = "\nServerID\tServerName\tEnabled";
			System.out.println(strMsg);
			HashSet servers = mgr.serverListIDs();
			if (servers.isEmpty()) {
				throw new FinderException();
			}
			Iterator it = servers.iterator();
			while (it.hasNext()) {
				Integer sid = (Integer) it.next();
				String sname = mgr.serverGetName(sid);
				boolean isEnabled = mgr.serverIsEnabled(sid);

				strMsg = sid + "\t" + sname + "\t" + isEnabled;
				System.out.println(strMsg);
			}
		} catch (NamingException e) {
			// jndiContext.lookup failed
			strMsg = "ERROR - Cannot find " + strObjName;
			logger.log(Level.WARNING, strMsg, e);
		} catch (RemoteException e) {
			// mgrHome.create failed
			strMsg = "ERROR - Cannot instantiate " + strObjName;
			logger.log(Level.WARNING, strMsg, e);
		} catch (CreateException e) {
			// mgrHome.create failed
			strMsg = "ERROR - Cannot instantiate " + strObjName;
			logger.log(Level.WARNING, strMsg, e);
		} catch (InvalidServerException e) {
			// failed mgr calls
			strMsg = "ERROR - InvalidServerException " + strObjName;
			logger.log(Level.WARNING, strMsg, e);
		} catch (FinderException e) {
			// failed mgr.serverListIDs
			System.out.println("...No Servers to List...");
		}

		// List all Replicas & their status
		System.out.println("Active Replicas:");
		if (replicas.size() == 0) {
			System.out.println("...None...");
		} else {
			strMsg = "ServerID\tServerName\tisAlive";
			System.out.println(strMsg);
			for (int i = 0; i < replicas.size(); i++) {
				ReplicaMonitor rm = (ReplicaMonitor) replicas.get(i);
				int sid = rm.getServerId();
				String sname = rm.getServerName();
				boolean isAlive = rm.isAlive();
				strMsg = sid + "\t" + sname + "\t" + isAlive;
				System.out.println(strMsg);
			}
		}
	}

	private void ServerCreate() {
		ManagerRemote mgr;
		String strObjName = "ManagerHomeRemote";
		String strMsg;

		// Get the Remote ManagerBean
		try {
			Object ref = jndiContext.lookup(strObjName);
			ManagerHomeRemote home =
				(ManagerHomeRemote) PortableRemoteObject.narrow(
																ref,
																ManagerHomeRemote.class);
			mgr = home.create();
		} catch (NamingException e) {
			// failed jndiContext.lookup
			strMsg = "ERROR - Cannot find " + strObjName;
			logger.log(Level.WARNING, strMsg, e);
			return;
		} catch (RemoteException e) {
			// failed home.create
			strMsg = "ERROR - Unable to instantiate " + strObjName;
			logger.log(Level.WARNING, strMsg, e);
			return;
		} catch (CreateException e) {
			// failed home.create
			strMsg = "ERROR - Unable to instantiate " + strObjName;
			logger.log(Level.WARNING, strMsg, e);
			return;
		}

		// Read the user Input for Server Attributes
		BufferedReader in =
			new BufferedReader(new InputStreamReader(System.in));

		System.out.println("Please input the server attributes");
		try {
			System.out.print("\tserverID:");
			Integer serverID = Integer.valueOf(in.readLine());
			System.out.print("\tserverName:");
			String serverName = in.readLine();
			mgr.serverCreate(serverID, serverName, false);
			strMsg = "Created Server[" + serverID + "]:" + serverName 
				+ "\tNew Servers Created as Disabled."
				+ "\nTo Start a Replica on the server, you must enable it.";
			System.out.println(strMsg);
		} catch (IOException e) {
			// failed in.readLine
			System.out.println("ERROR - Unable to read input");
			return;
		} catch (NumberFormatException e) {
			// failed Integer.valueOf
			System.out.println("ERROR - Invalid number");
			return;
		} catch (CreateException e) {
			// failed mgr.serverCreate
			strMsg = "ERROR - Unable to create server";
			logger.log(Level.WARNING, strMsg, e);
			return;
		} catch (DuplicateServerException e) {
			strMsg = "ERROR - Duplicate ServerID";
			System.out.println("\t" + strMsg);
			return;
		}
	}

	private void ServerRemove() {
		ManagerRemote mgr;
		String strObjName = "ManagerHomeRemote";
		String strMsg;

		// Get the Remote ManagerBean
		try {
			Object ref = jndiContext.lookup(strObjName);
			ManagerHomeRemote home =
				(ManagerHomeRemote) PortableRemoteObject.narrow(
																ref,
																ManagerHomeRemote.class);
			mgr = home.create();
		} catch (NamingException e) {
			// failed jndiContext.lookup
			strMsg = "ERROR - Cannot find " + strObjName;
			logger.log(Level.WARNING, strMsg, e);
			return;
		} catch (RemoteException e) {
			// failed home.create
			strMsg = "ERROR - Unable to instantiate " + strObjName;
			logger.log(Level.WARNING, strMsg, e);
			return;
		} catch (CreateException e) {
			// failed home.create
			strMsg = "ERROR - Unable to instantiate " + strObjName;
			logger.log(Level.WARNING, strMsg, e);
			return;
		}

		// For convenience to user, show list of servers...
		ServerList();

		// Read the user for which Server to remove
		BufferedReader in =
			new BufferedReader(new InputStreamReader(System.in));

		System.out.print("\tEnter the serverID to remove:");
		try {
			Integer serverID = Integer.valueOf(in.readLine());

			// Verify the server is not enabled.
			// - The user must first disable a server before removing
			//   it, to avoid active ReplicaMonitor threads polling
			//   the disabled server.
			if( mgr.serverIsEnabled(serverID) ) {
				strMsg = "ERROR - Server is Enabled!"
					+ "\tYou must disable the server before removing it.";
				logger.log(Level.WARNING, strMsg);
				return;
			}

			// Reuqest the user to confirm.
			System.out.print("\tAre you sure? (y/N):");
			switch (in.readLine().toUpperCase().charAt(0)) {
				case 'Y' :
					strMsg = "Server[" + serverID + "] Removed...";
					mgr.serverRemove(serverID);
					logger.info(strMsg);
					break;
				default :
					System.out.println("Cancelled Server Removal...");
			}
		} catch (IOException e) {
			// failed in.readLine
			System.out.println("ERROR - Unable to read input");
		} catch (StringIndexOutOfBoundsException e) {
			// User probably just hit "enter" without providing input.
			System.out.println("ERROR - Missing input");
		} catch (NumberFormatException e) {
			// failed Integer.valueOf
			System.out.println("ERROR - Invalid number");
		} catch (InvalidServerException e) {
			// failed mgr.serverRemove
			strMsg = "ERROR - Invalid ServerID";
			System.out.println("\t" + strMsg);
		} catch (RemoveException e) {
			// failed mgr.serverRemove
			strMsg = "ERROR - Unable to remove Server";
			logger.log(Level.WARNING, strMsg, e);
		}
	}

	private void ServerRun(boolean enable) {
		// For user convenience, list the servers
		ServerList();

		// Prompt the user for a serverID
		String str = enable ? "Start:" : "Stop:";
		String strMsg = "\tEnter ServerID to " + str + " ('-1':=ALL):";
		System.out.print(strMsg);

		// Read user input
		try {
			BufferedReader in =
				new BufferedReader(
						new InputStreamReader(System.in));
			String choice = in.readLine();
			Integer serverID = new Integer(choice);
			ServerStartStop(serverID, enable);
		} catch (IOException e) {
			strMsg = "Error reading input";
			logger.log(Level.WARNING, strMsg, e);
		}
	}

	private void ServerStartStop(Integer serverID, boolean StartStop) {
		ManagerRemote mgr;
		String strObjName = "ManagerHomeRemote";
		String strMsg;

		strMsg = "ServerID[" + serverID + "] " + ( StartStop ? "Start" : "Stop");
		System.out.println(strMsg);

		// Get the Remote ManagerBean
		try {
			Object ref = jndiContext.lookup(strObjName);
			ManagerHomeRemote home =
				(ManagerHomeRemote) PortableRemoteObject.narrow(
																ref,
																ManagerHomeRemote.class);
			mgr = home.create();
		} catch (NamingException e) {
			// failed jndiContext.lookup
			strMsg = "ERROR - Cannot find " + strObjName;
			logger.log(Level.WARNING, strMsg, e);
			return;
		} catch (RemoteException e) {
			// failed home.create
			strMsg = "ERROR - Unable to instantiate " + strObjName;
			logger.log(Level.WARNING, strMsg, e);
			return;
		} catch (CreateException e) {
			// failed home.create
			strMsg = "ERROR - Unable to instantiate " + strObjName;
			logger.log(Level.WARNING, strMsg, e);
			return;
		}

		// Verify the serverID
		if (serverID.intValue() == -1) { // Start or Stop ALL Servers
			if (StartStop) {
				// Iterate over Enabled Servers ...
				// ... and start ReplicaMonitors for them.
				System.out.println("\nLauching ALL enabled Servers...");
				try {
					HashSet servers = mgr.serverListIDs();
					if (servers.isEmpty()) {
						throw new FinderException();
					}
					Iterator it = servers.iterator();
					while (it.hasNext()) {
						Integer sid = (Integer) it.next();
						ServerStartStop(sid, StartStop);
					}
				} catch (FinderException e) {
					System.out.println("ERROR - No Enabled Servers act upon.");
					return;
				} catch( RemoteException e) {
					// failed mgr call
					strMsg = "ERROR - communication failure with " + strObjName;
					logger.log(Level.WARNING, strMsg, e);
					return;
				}
				this.setNumServers(replicas.size());
				strMsg = "Done - All enabled servers launched.";
				System.out.println(strMsg);
			} else {
				// Iterate over Active ReplicaMontitors ...
				// ... and Terminate them.
				System.out.println("\nTerminating ALL Replicas...");
				if( replicas.size() == 0) {
					System.out.println("...No Replicas to Terminate...");
				}
				for (int idx = 0; idx < replicas.size(); idx++) {
					ReplicaMonitor rm = (ReplicaMonitor) replicas.get(idx);
					System.out.println((idx + 1) + ") " + rm.getServerName());
					rm.halt();
				}
				strMsg = "Done - All enabled replicas terminated.";
				System.out.println(strMsg);
			}
		} else {
			// Check if ReplicaMonitor already running for this serverID.
			ReplicaMonitor rm = null;
			for (int idx = 0; idx < replicas.size(); idx++) {
				ReplicaMonitor rmTmp = (ReplicaMonitor) replicas.get(idx);
				if (rmTmp.getServerId() == serverID.intValue()) {
					rm = rmTmp; // Found one!
					break;
				}
			}

			try {
				if (StartStop) {
					String strName;

					// Enable the Server
					strName = mgr.serverGetName(serverID);
					strMsg = "Server[" + serverID +"]:" + strName +" enabled";
					logger.info(strMsg);
					mgr.serverSetEnabled(serverID, false);  // don't use yet

					// If ReplicaMonitor is already running...
					// ...then nothing to do.
					if (rm != null) {
						System.out.println("WARNING - Server Replica already running...");
						return;
					}

					// Start a ReplicaMonitor for the indicated server.
					rm = new ReplicaMonitor(
							serverID.intValue(),
							strName,
							this.getCheckPeriod(),
							this, mgr);

					// careful - the indexes might get out of order somehow
					rm.start();
					replicas.add(rm);

				} else {
					String strName;

					// If a ReplicaMonitor is running, then kill it.
					// - This terminates ReplicaMonitor Thread
					if (rm != null) {
						rm.halt();
					}
					replicas.remove(rm);

					// Disable the server (for maintainence)
					strName = mgr.serverGetName(serverID);
					strMsg = "Server[" + serverID +"]:" + strName +" disabled";
					logger.info(strMsg);
					mgr.serverSetEnabled(serverID, false);  // don't use yet
				}
			} catch( RemoteException e) {
				// failed mgr call
				strMsg = "ERROR - communication failure with " + strObjName;
				logger.log(Level.WARNING, strMsg, e);
				return;
			} catch( InvalidServerException e) {
				// failed mgr.serverGetName/IsEnabled
				strMsg = "ERROR - Invalid serverID:" + serverID;
				logger.log(Level.WARNING, strMsg);
				return;
			}
		}
	}

	private void ClientList() {
		String strObjName = "ManagerHomeRemote";
		String strMsg;

		try {
			// Get the Remote ManagerBean
			Object ref = jndiContext.lookup(strObjName);
			ManagerHomeRemote home =
				(ManagerHomeRemote) PortableRemoteObject.narrow(
																ref,
																ManagerHomeRemote.class);
			ManagerRemote mgr = home.create();

			// List set of all known Clients
			strMsg = "\nClientID\tClientName\tEnabled";
			System.out.println(strMsg);
			HashSet clients = mgr.clientListIDs();
			if (clients.isEmpty()) {
				throw new FinderException();
			}
			Iterator it = clients.iterator();
			while (it.hasNext()) {
				Integer cid = (Integer) it.next();
				String cname = mgr.clientGetName(cid);
				// boolean isEnabled = mgr.clientIsEnabled(cid);

				strMsg = cid + "\t" + cname + "\t ?";
				System.out.println(strMsg);
			}
		} catch (NamingException e) {
			// jndiContext.lookup failed
			strMsg = "ERROR - Cannot find " + strObjName;
			logger.log(Level.WARNING, strMsg, e);
		} catch (RemoteException e) {
			// mgrHome.create failed
			strMsg = "ERROR - Cannot instantiate " + strObjName;
			logger.log(Level.WARNING, strMsg, e);
		} catch (CreateException e) {
			// mgrHome.create failed
			strMsg = "ERROR - Cannot instantiate " + strObjName;
			logger.log(Level.WARNING, strMsg, e);
		} catch (InvalidClientException e) {
			// failed mgr calls
			strMsg = "ERROR - InvalidClientException " + strObjName;
			logger.log(Level.WARNING, strMsg, e);
		} catch (FinderException e) {
			// failed mgr.serverListIDs
			System.out.println("...No Servers to List...");
		}
	}

	private void ClientCreate() {
		ManagerRemote mgr;
		String strObjName = "ManagerHomeRemote";
		String strMsg;

		// Get the Remote ManagerBean
		try {
			Object ref = jndiContext.lookup(strObjName);
			ManagerHomeRemote home =
				(ManagerHomeRemote) PortableRemoteObject.narrow(
																ref,
																ManagerHomeRemote.class);
			mgr = home.create();
		} catch (NamingException e) {
			// failed jndiContext.lookup
			strMsg = "ERROR - Cannot find " + strObjName;
			logger.log(Level.WARNING, strMsg, e);
			return;
		} catch (RemoteException e) {
			// failed home.create
			strMsg = "ERROR - Unable to instantiate " + strObjName;
			logger.log(Level.WARNING, strMsg, e);
			return;
		} catch (CreateException e) {
			// failed home.create
			strMsg = "ERROR - Unable to instantiate " + strObjName;
			logger.log(Level.WARNING, strMsg, e);
			return;
		}

		// Read the user Input for Client Attributes
		BufferedReader in =
			new BufferedReader(new InputStreamReader(System.in));

		System.out.println("Please input the client attributes");
		try {
			System.out.print("\tclientID:");
			Integer cID = Integer.valueOf(in.readLine());
			System.out.print("\tclientName:");
			String cName = in.readLine();
			//System.out.print("\tclientEnable:");
			//Boolean cEnable = Boolean.valueOf(in.readLine());
			mgr.clientCreate(cID, cName);
			strMsg = "Created Client[" + cID + "]:" + cName;
			//strMsg = strMsg + " Enabled:" + cEnable;
			System.out.println(strMsg);
		} catch (IOException e) {
			// failed in.readLine
			System.out.println("ERROR - Unable to read input");
			return;
		} catch (NumberFormatException e) {
			// failed Integer.valueOf
			System.out.println("ERROR - Invalid number");
			return;
		} catch (CreateException e) {
			// failed mgr.serverCreate
			strMsg = "ERROR - Unable to create server";
			logger.log(Level.WARNING, strMsg, e);
			return;
		} catch (DuplicateClientException e) {
			strMsg = "ERROR - Duplicate ClientID";
			System.out.println("\t" + strMsg);
			return;
		}
	}

	private void ClientRemove() {
		ManagerRemote mgr;
		String strObjName = "ManagerHomeRemote";
		String strMsg;

		// Get the Remote ManagerBean
		try {
			Object ref = jndiContext.lookup(strObjName);
			ManagerHomeRemote home =
				(ManagerHomeRemote) PortableRemoteObject.narrow(
																ref,
																ManagerHomeRemote.class);
			mgr = home.create();
		} catch (NamingException e) {
			// failed jndiContext.lookup
			strMsg = "ERROR - Cannot find " + strObjName;
			logger.log(Level.WARNING, strMsg, e);
			return;
		} catch (RemoteException e) {
			// failed home.create
			strMsg = "ERROR - Unable to instantiate " + strObjName;
			logger.log(Level.WARNING, strMsg, e);
			return;
		} catch (CreateException e) {
			// failed home.create
			strMsg = "ERROR - Unable to instantiate " + strObjName;
			logger.log(Level.WARNING, strMsg, e);
			return;
		}

		// For convenience to user, show list of clients...
		ClientList();

		// Read the user for which Server to remove
		BufferedReader in =
			new BufferedReader(new InputStreamReader(System.in));

		System.out.print("\tEnter the ClientID to remove:");
		try {
			Integer cID = Integer.valueOf(in.readLine());

			// Reuqest the user to confirm.
			String cName = mgr.clientGetName(cID);
			System.out.print("\tAre you sure? (y/N):");
			switch (in.readLine().toUpperCase().charAt(0)) {
				case 'Y' :
					strMsg = "Client[" + cID + "]:" + cName + " Removed.";
					mgr.clientRemove(cID);
					logger.info(strMsg);
					break;
				default :
					System.out.println("Cancelled Client Removal...");
			}
		} catch (IOException e) {
			// failed in.readLine
			System.out.println("ERROR - Unable to read input");
		} catch (StringIndexOutOfBoundsException e) {
			// User probably just hit "enter" without providing input.
			System.out.println("ERROR - Missing input");
		} catch (NumberFormatException e) {
			// failed Integer.valueOf
			System.out.println("ERROR - Invalid number");
		} catch (InvalidClientException e) {
			// failed mgr.serverRemove
			strMsg = "ERROR - Invalid ClientID";
			System.out.println("\t" + strMsg);
		} catch (RemoveException e) {
			// failed mgr.serverRemove
			strMsg = "ERROR - Unable to remove Server";
			logger.log(Level.WARNING, strMsg, e);
		}
	}

	/**
	 * Can't yet change the number of servers after start up.
	 * 
	 * @param num
	 */
	private void setNumServers(int num) {
		this.numServers = num;
	}

	public int getNumServers() {
		return this.numServers;
	}

	public int getCheckPeriod() {
		return this.sleepPeriod;
	}

	public void setCheckPeriod(int secs) {
		this.sleepPeriod = secs;
		for (int i = 0; i < replicas.size(); i++) {
			ReplicaMonitor rm = (ReplicaMonitor) (replicas.get(i));
			rm.setCheckPeriod(secs);
		}
	}

}
