// $Id: ReplicaStateBean.java,v 1.4 2004/03/05 23:05:10 gca Exp $

package com.spamnbeans.server;

import javax.ejb.SessionBean;

import javax.ejb.SessionContext;

/**
 * The ReplicaState
 * 
 * <p>
 * The ReplicaState Bean is monitored by the GlobalReplicationManager to make
 * sure the replica is still alive, and to determine the load from client
 * requests.
 * 
 * @author Gary Ackley, 
 *         <a href="mailto:gca@andrew.cmu.edu">gca@andrew.cmu.edu</a>
 * @version $Revision: 1.4 $
 */
public class ReplicaStateBean implements SessionBean {
	private SessionContext ctx;
	
	// Private variable to hold the number of client requests
	// - Is incremented by local MailProc.processMessage
	// - Is queried & cleared by remote ReplicaMonitor
	private static long NbrRequests = 0;

	/**
	 * Return the number of requests of the currently connected Server
	 * since last call 
	 */
	public long getNbrRequests() {
		long retval = NbrRequests;
		// Clear counter for next time
		NbrRequests = 0;
		return retval;
	}

	/**
	 * Called by each MailProc to indicate a client request has completed.
	 */
	public void addRequest() {
		// TODO Confirm there is no need for a mutex here
		// - I think Container handles mutex.
		NbrRequests++;
	}

	/*
	 * EJB Required Methods...
	 */
	
	public void ejbCreate() {
		System.out.println("ReplicaState : ejbCreate");
	}

	public void ejbRemove() {
		System.out.println("ReplicaState : ejbRemove");
	}

	public void ejbActivate() {
		System.out.println("ReplicaState : ejbActivate");
	}

	public void ejbPassivate() {
		System.out.println("ReplicaState : ejbPassivate");
	}

	public void setSessionContext(SessionContext ctx) {
		System.out.println("ReplicaState : setSessionContext");
		this.ctx = ctx;
	}
}
